@@ -7,6 +7,20 @@ pd.options.display.max_rows = 100
 
 # fetch metadata
 
+def ner_from_urn(urn=None, model=None):
+    """Get a NER object for a text by URN using a spacy model
+    :param urn: a URN
+    :param model: a spacy model (check with show_model what is available)"""
+    
+    params = locals()
+    r = requests.get(f"{BASE_URL}/ner_urn", params=params)
+    df = pd.read_json(r.json())
+    return df
+
+def show_spacy_models():
+    """Show available models for use with spaCy"""
+    r = requests.get(f"{BASE_URL}/ner_models")
+    return r.json()
 
 def get_places(urn=None) -> pd.DataFrame:
     """
@@ -34,6 +48,15 @@ def get_dispersion(urn=None, words=None, window=None, pr=None) -> pd.Series:
     r = requests.post(f"{BASE_URL}/dispersion", json=params)
     return pd.Series(r.json())
 
+def ner_from_urn(urn=None, model=None):
+    """Get a NER object for a text by URN using a spacy model
+    :param urn: a URN
+    :param model: a spacy model (check with show_model what is available)"""
+    
+    params = locals()
+    r = requests.get(f"{BASE_URL/ner_urn}", params=params)
+    return 
+                     
 
 def get_metadata(urns=None) -> pd.DataFrame:
     """Fetch metadata from a list of urns."""
@@ -200,25 +223,37 @@ def ngram_news(word=['.'], title=None, period=None):
     #df.index = df.index.map(pd.Timestamp)
     return df
 
-
-def get_document_frequencies(urns=None, cutoff=0, words=None):
+def get_document_frequencies(urns=None, cutoff=0, words = None):
+    """Get frequency numbers from documents as a list of URNs
+    :urns: list of urns
+    :cutoff: minimum frequency of a word to be counted
+    :word: a list of words to be counted - if left None, whole document is returned"""
     params = locals()
-    r = requests.post(BASE_URL + "/frequencies", json=params)
+    r = requests.post(f"{BASE_URL}/frequencies", json=params)
     result = r.json()
-    structure = {u[0][0]: dict([tuple(x[1:3]) for x in u])
-                 for u in result if u != []}
-    df = pd.DataFrame(structure)
-    return df.sort_values(by=df.columns[0], ascending=False)
-
+    # check if words are passed - return differs a bit
+    # check if words are passed - return differs a bit
+    if words is None:
+        structure = dict()
+        for u in result:
+            try:
+                structure[u[0][0]] = dict([(x[1],x[2]) for x in u ]) 
+            except IndexError:
+                pass
+        df = pd.DataFrame(structure)
+        df = df.sort_values(by=df.columns[0], ascending=False).fillna(0)
+    else:     
+        df = pd.DataFrame(result)
+        df.columns = ["urn", "word", "count", "urncount"]  
+        df = pd.pivot_table(df,  values = 'count', index = 'word', columns = "urn").fillna(0)
+    return df
 
 def get_word_frequencies(urns=None, cutoff=0, words=None):
-    params = locals()
-    r = requests.post(BASE_URL + "/frequencies", json=params)
-    result = r.json()
-    structure = {u[0][0]: dict([(x[1], x[2] / x[3]) for x in u])
-                 for u in result if u != []}
-    df = pd.DataFrame(structure)
-    return df.sort_values(by=df.columns[0], ascending=False)
+    """Get frequency numbers from documents as a list of URNs 
+    :urns: list of urns
+    :cutoff: minimum frequency of a word to be counted
+    :word: a list of words to be counted - should not be left None"""
+    return get_document_frequencies(urns, cutoff, words)
 
 
 def get_document_corpus(**kwargs):
